/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.52a                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for NEC D70F37xxGJ and NEC compiler)
Purpose : BSP for NEC AF-EV850 Basic development board
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/


/****** SFRs and identifiers used for LED demo **********************/

/****** Assign LEDs to Ports ****************************************/
#define _LED0_BIT         1
#define _LED1_BIT         2

#define P6                *((volatile unsigned int*)0xFFFFF40C)
#define PF6               *((volatile unsigned char*)0xFFFFF46D)
#define PM6               *((volatile unsigned char*)0xFFFFF42C)
#define PMC6              *((volatile unsigned char*)0xFFFFF44C)
#define P9                *((volatile unsigned int*)0xFFFFF412)
#define PF9               *((volatile unsigned char*)0xFFFFF472)
#define PM9               *((volatile unsigned char*)0xFFFFF432)
#define PMC9              *((volatile unsigned char*)0xFFFFF452)
#define _LEDPORT_STATE    P9


/****** Target specific configuration *******************************/


/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/

void BSP_Init(void) {
  PM6 = 0x00;      /* Set all pins to output                                */
  PM9 = 0x00;      /* Set all pins to output                                */
  PMC6 = 0x00;     /* Set port pin mode                                     */
  PMC9 = 0x00;     /* Set port pin mode                                     */
  P6 = 0xFFFE;     /* Switch off all "leds" (segments of 7-segment display) */
  P9 = 0xFFFF;
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with high level on port lines
*/

void BSP_SetLED(int Index) {
  _LEDPORT_STATE &= ~(1 << (_LED0_BIT + Index));           /* Switch on LED         */
}

void BSP_ClrLED(int Index) {
  _LEDPORT_STATE |= (1 << (_LED0_BIT + Index));            /* Switch off LED        */
}

void BSP_ToggleLED(int Index) {
  if (_LEDPORT_STATE & (1 << (_LED0_BIT + Index))) {       /* Toggle LED            */
    BSP_SetLED(Index);
  } else {
    BSP_ClrLED(Index);
    
  }
}

/****** EOF *********************************************************/

