#   Copyright (C) NEC Electronics Corporation 1998,2002
#   NEC ELECTRONICS CONFIDENTIAL AND PROPRIETARY
#   All rights reserved by NEC Electronics Corporation.
#   This program must be used solely for the purpose for which
#   it was furnished by NEC Electronics Corporation.  No part of this
#   program may be reproduced or disclosed to others, in any
#   form, without the prior written permission of NEC Electronics
#   Corporation.  Use of copyright notice does not evidence
#   publication of the program.

#	@(#)crtE.s	1.8 02/12/12 15:19:37

#=============================================================================
# NAME
#    crtE.s -  start up module for ca850(V850E)
#
# DESCRIPTIONS:
#      This assembly program is a sample of start-up module for ca850(V850E).
#    If you modified this program, you must assemble this file, and
#    locate a given directory.
#
#    Unless -G is specified, sections are located as the following.
#
#                  |        :        |
#                  |        :        |
#	        tp -> -+-----------------+ __start     __tp_TEXT
#                  |  start up       |
#                  |---------------- |
#   text section   |                 |
#                  | user program    |
#                  |                 |
#                  |-----------------|
#                  | library         |
#	              -+-----------------+ 
#                  |        :        |
#                  |        :        |
#	              -+-----------------+ __argc
#                  |      0          |
#                  |---------------- | __argv
#   data section   |    #.L16        |
#                  |---------------- | .L16
#                  | 0x0,0x0,0x0,0x0 |
#                 -+-----------------+               
#                  |                 |
#   sdata section  |                 |
#                  |                 |
#            gp-> -+-----------------+             __ssbss
#                  |                 |
#   sbss section   |                 |
#                  |                 |
#                  +-----------------+ __stack     __esbss     __sbss
#                  | stack area      |
#   bss section    |                 |
#                  |  0x200 bytes    |
#            sp-> -+-----------------+ __stack + STACKSIZE     __ebss
#
#=============================================================================


#-----------------------------------------------------------------------------
#	special symbols
#-----------------------------------------------------------------------------
	.extern __tp_TEXT, 4
	.extern __gp_DATA, 4
	.extern __ep_DATA, 4
	.extern	__ssbss, 4
	.extern __esbss, 4
	.extern __sbss, 4
	.extern __ebss, 4

    .globl  __stackend
    .globl  __stack
	
#-----------------------------------------------------------------------------
#	 C program main function
#-----------------------------------------------------------------------------
	.extern	_main


#-----------------------------------------------------------------------------
#	for argv
#-----------------------------------------------------------------------------
	.data
	.size	__argc, 4
	.align	4
__argc:
	.word	0
	.size	__argv, 4
__argv:
	.word	#.L16
.L16:
	.byte	0
	.byte	0
	.byte	0
	.byte	0


#-----------------------------------------------------------------------------
#	dummy data declaration for creating sbss section
#-----------------------------------------------------------------------------
	.sbss
	.lcomm	__sbss_dummy, 0, 0

	
#-----------------------------------------------------------------------------
#	system stack
#-----------------------------------------------------------------------------
	.set	STACKSIZE, 0x200
	.bss
	.lcomm	__stack, STACKSIZE, 4
	.lcomm	__stackend, 0x00, 4
	


#-----------------------------------------------------------------------------
#	RESET handler
#-----------------------------------------------------------------------------

	.section        "RESET", text
	jr      __start
	
#-----------------------------------------------------------------------------
#	start up
#		pointers:  tp - text pointer
#			   gp - global pointer
#			   sp - stack pointer
#			   ep - element pointer
#		mask reg:  r20 - 0xff
#			   r21 - 0xffff
#	exit status is set to r10
#-----------------------------------------------------------------------------
	.text
	.align	4
	.globl	__start
	.globl  __exit
	.globl	__startend
	.extern  ___PROLOG_TABLE


#-----------------------------------------------------------------------------
#
#  Abstract:
#	Init the Clock Generator and Watchdog timer 2
#
#---------------------------------------------------------------------------
    .globl _PLL_Init
_PLL_Init:
	add	 -8, sp
	st.w	r11, 0[sp]
	st.w	r12, 4[sp]

	ld.b	DCHC0[r0], r11			-- stop DMA0
	add	-1, sp
	st.b	r11, 0[sp]
	andi	0xfe, r11, r11
	st.b	r11, DCHC0[r0]
	
	ld.b	DCHC1[r0], r11			-- stop DMA1
	add	-1, sp
	st.b	r11, 0[sp]
	andi	0xfe, r11, r11
	st.b	r11, DCHC1[r0]
	
	ld.b	DCHC2[r0], r11			-- stop DMA2
	add	-1, sp
	st.b	r11, 0[sp]
	andi	0xfe, r11, r11
	st.b	r11, DCHC2[r0]
	
	ld.b	DCHC3[r0], r11			-- stop DMA3
	add	-1, sp
	st.b	r11, 0[sp]
	andi	0xfe, r11, r11
	st.b	r11, DCHC3[r0]
	
	-- disable interrupt
	stsr	5, r11
	ori	0xa0, r11, r11
	ldsr	r11, 5

	mov	r0, r11
	st.b	r11, PRCMD[r0]
	st.b	r11, CLM[r0]			--disable clock monitor function

	nop
	nop
	nop
	nop
	nop
	ld.b	PCC[r0], r12
	andi	0xf8, r12, r12
	or		r12, r11
	st.b	r11, PRCMD[r0]
	st.b	r11, PCC[r0]

	nop
	nop
	nop
	nop
	nop
	-- Sub clock -> Main clock start
	-- stop Main clock
	st.b	r0, PRCMD[r0]
	clr1	6, PCC[r0]
	-- this wait loop per 250usec
	movea	0x1000, r0, r11

	__CG_LOOP2:
	nop
	nop
	nop

	addi	-1, r11, r11
	cmp	r0, r11
	bnz	__CG_LOOP2
	st.b	r0, PRCMD[r0]
	
	clr1	3, PCC[r0]
	__CG_LOOP3:
	-- Check CLS
	tst1	4, PCC[r0]
	bnz	__CG_LOOP3

	-- Sub -> Main  end
	-- enable RingOSC
	clr1	0, RCM[r0]
	mov	0x0a, r11			-- fxx = 4*fx
	st.b	r11, PRCMD[r0]
	st.b	r11, CKC[r0]
	nop
	nop
	nop
	nop
	nop
	-- PLL start
	set1	0, PLLCTL[r0]
	-- PLL work
	__CG_LOOP4:
	ld.b	LOCKR[r0], r11
	cmp	r0, r11
	bnz	__CG_LOOP4
	set1	1, PLLCTL[r0]
	-- enable interrupt
	stsr	5, r11
	andi	0x5f, r11, r11
	ldsr	r11, 5

	ld.b	0[sp], r11			-- recover DMA3
	add	1, sp
	st.b	r11, DCHC3[r0]
	
	ld.b	0[sp], r11			-- recover DMA2
	add	1, sp
	st.b	r11, DCHC2[r0]
	
	ld.b	0[sp], r11			-- recover DMA1
	add	1, sp
	st.b	r11, DCHC1[r0]
	
	ld.b	0[sp], r11			-- recover DMA0
	add	1, sp
	st.b	r11, DCHC0[r0]
	-- oscollation stabilization time
	-- selection clock
	-- 2^16/fx
	mov	0x6, r11
	st.b	r11, OSTS[r0]
	mov	0x1f, r11
	st.b	r11, WDTM2[r0]
	-- pop
	ld.w	0[sp], r11
	ld.w	4[sp], r12
	add	8, sp
	
	jmp	[lp]




__start:
	mov	#__tp_TEXT, tp		-- set tp register
	mov	#__gp_DATA, gp		-- set gp register offset
	add	tp, gp			-- set gp register
	mov	#__stack+STACKSIZE, sp	-- set sp register
	mov	#__ep_DATA, ep		-- set ep register
#
	.option nowarning
	mov	0xff,   r20		-- set mask register
	mov	0xffff, r21		-- set mask register
	.option warning
#
	mov	#__ssbss, r13		-- clear sbss section
	mov	#__esbss, r12
	cmp	r12, r13
	jnl	.L11
.L12:
	st.w	r0, [r13]
	add	4, r13
	cmp	r12, r13
	jl	.L12
.L11:
#
	mov	#__sbss, r13		-- clear bss section
	mov	#__ebss, r12
	cmp	r12, r13
	jnl	.L14
.L15:
	st.w	r0, [r13]
	add	4, r13
	cmp	r12, r13
	jl	.L15
.L14:
#
	mov     #___PROLOG_TABLE, r12   -- for prologue/epilogue runtime
	ldsr    r12, 20			-- set CTBP (CALLT base pointer)
#
#					-- for Programmable peripheral I/O 
#	mov	0x9234, r13		-- 0x1234(addr) | 0x8000(use pI/Or)
#	st.h	r13, BPC		-- set BPC
#	

	ld.w	$__argc, r6		-- set argc
	movea	$__argv, gp, r7		-- set argv
	jarl	_main, lp		-- call main function
__exit:
	halt				-- end of program
__startend:
#								#
#-------------------- end of start up module -------------------#
#								#

